<?php

namespace Drupal\power_reviews\Controller;

/**
 * Controller routines for power_reviews routes.
 */
class PowerReviewsController {

  /**
   * Returns content for Write A Review page.
   *
   * @return array
   *   A render array representing the content.
   */
  public function writeReview() {
    $current_language = \Drupal::languageManager()->getCurrentLanguage()->getId();
    $nid = \Drupal::request()->query->get('nid');
    $review_id = \Drupal::config('power_reviews.settings')->get('power_reviews_write_review');
    $power_reviews_type_of_implementation = \Drupal::config('power_reviews.settings')->get('power_reviews_type_of_implementation');
    $node_types = \Drupal::config('power_reviews.settings')->get('power_reviews_applied_content_types');
    $node_types = array_values($node_types);
    $node_types = array_map('trim', $node_types);

    if (\Drupal::config('power_reviews.settings')->get('power_reviews_feedless_implementation') == 'yes') {
      $product_details = [];
      if ($power_reviews_type_of_implementation == 'page_path') {
        $query = new EntityFieldQuery();
        $query->entityCondition('entity_type', 'node', '=');
        $query->propertyCondition('type', $node_types);
        $query_result = $query->execute();

        if (isset($query_result['node'])) {
          $node_ids = array_keys($query_result['node']);
          $nodes = node_load_multiple($node_ids);
        }

        foreach ($nodes as $node) {
          $product_array = [];
          $power_reviews_field_names = json_decode(\Drupal::config('power_reviews.settings')->get('power_reviews_default_fields'), TRUE);
          foreach ($power_reviews_field_names as $field_name => $field_value) {
            if ($field_value['bundle'] == 'node') {
              $field_detail = $node->$field_name;
              if ($field_name == 'field_variants' && isset($field_detail[$current_language])) {
                foreach ($field_detail[$current_language] as $key => $fc_id) {
                  $fc_item_array = entity_load('paragraph', [$fc_id['value']]);
                  $fc_item = reset($fc_item_array);
                  foreach ($power_reviews_field_names as $fc_item_name => $fc_item_value) {
                    if ($fc_item_value['bundle'] == 'paragraph') {
                      $product_array[$field_value['power_review_key']][$key][$fc_item_value['power_review_key']] = (isset($fc_item->{$fc_item_name}[$current_language])) ?
                      $fc_item->{$fc_item_name}[$current_language][0]['value'] : '';
                    }
                  }
                }
              }
              elseif ($field_name == 'field_power_reviews_image_url') {
                $product_array[$field_value['power_review_key']] = (isset($field_detail[$current_language])) ? file_create_url($field_detail[$current_language][0]['uri']) : '';
              }
              else {
                $product_array[$field_value['power_review_key']] = (isset($field_detail[$current_language])) ? $field_detail[$current_language][0]['value'] : '';
              }
            }
          }

          $product_array['amazon_link'] = (isset($node->field_amazon_link[$current_language])) ? $node->field_amazon_link[$current_language][0]['url'] : '';
          $product_details[$node->nid] = $product_array;
        }
      }
      else {
        if (is_numeric($nid)) {
          $product = node_load($nid);
          $product_array = [
            'name' => (NULL !== $product->getTitle()) ? $product->getTitle() : '',
            'sub_title' => !$product->get('field_product_sub_title')->isEmpty() ? $product->field_product_sub_title->value : '',
            'image_url' => !$product->get('field_power_reviews_image_url')->isEmpty() ? file_create_url($product->get('field_power_reviews_image_url')->entity->getFileUri()) : '',
            'color' => !$product->get('field_product_color')->isEmpty() ? $product->get('field_product_color')->getValue()[0]['color'] : '',
          ];
          $product_details[$product->nid->value] = $product_array;
        }
      }
    }

    if ($power_reviews_type_of_implementation == 'page_path') {
      $powerreviews_page_id = \Drupal::config('power_reviews.settings')->get('power_reviews_page_id');
    }
    else {
      $powerreviews_page_id = \Drupal::request()->query->get('pr_page_id');
    }

    $powerreview_api_details = power_reviews_initiate();
    $powerreviews_review_details = [
      'powerreviews_page_id' => $powerreviews_page_id,
      'power_reviews_write_review' => $review_id,
      'power_reviews_write_review_custom_header' => \Drupal::config('power_reviews.settings')->get('power_reviews_write_review_custom_header'),
      'product_details' => $product_details,
    ];

    if (\Drupal::config('power_reviews.settings')->get('power_reviews_write_review_custom_header') == "yes") {
      $review_rtn_array = [
        'power_reviews' => [
          'review_id' => $review_id,
          'header_title' => \Drupal::config('power_reviews.settings')->get('power_reviews_write_review_custom_header_title'),
          'header_body' => \Drupal::config('power_reviews.settings')->get('power_reviews_write_review_custom_header_body'),
        ],
        'product_details' => $product_details,
      ];

      $build = [
        '#theme' => 'write_a_review',
        '#data' => $review_rtn_array,
      ];

      $build['#attached']['library'][] = 'power_reviews/power-reviews-js';
      $build['#attached']['library'][] = 'power_reviews/power-reviews-war-script';
      $build['#attached']['drupalSettings']['powerreview_api_details'] = $powerreview_api_details;
      $build['#attached']['drupalSettings']['powerreviews_review_details'] = $powerreviews_review_details;

      return $build;
    }
    else {
      return "<div id='" . $review_id . "'></div>";
    }
  }

}
